/*
	ChoicesCmd.cc

	Will Eighteen, 2001: will@opaline.demon.co.uk
*/

#include "ChoicesCmd.h"
#include "Application.h"
#include <MessageDialogManager.h>
#include <Xm/Protocols.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include "OpResources.h"


ChoicesCmd::ChoicesCmd ( const char *name, int active ) : NoUndoCmd ( name, active )
{
	strncpy ( _name, name, 128 );
	_popupRealised = False;
}


ChoicesCmd::~ChoicesCmd()
{
	// empty
}


void ChoicesCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	int n;
	Arg args[3];
	Atom WM_DELETE_WINDOW;

	_popup = XtCreatePopupShell ( _name, transientShellWidgetClass,
									application->baseWidget(), NULL, 0 );

	_rc = XtVaCreateManagedWidget ( "RowColumn", xmRowColumnWidgetClass, _popup,
									XmNorientation, XmVERTICAL,
									XmNnumColumns, 1, NULL );

	n = 0;	
	
	XtSetArg ( args[n], XmNtoggleMode, XmTOGGLE_BOOLEAN ); n++;
	XtSetArg ( args[n], XmNhighlightThickness, 0 ); n++;

	_syncLength = XtCreateManagedWidget ( NULL, xmToggleButtonWidgetClass, _rc, args, n );
	XtAddCallback ( _syncLength, XmNvalueChangedCallback, &ChoicesCmd::syncLengthCallback, ( XtPointer ) this );

	if ( mixwaveResources->syncTemplateLength() == True )
	{
		XmString xmstr = XmStringCreateSimple ( "Time sync.  ON" );
		XtVaSetValues ( _syncLength, XmNset, XmSET, XmNlabelString, xmstr, NULL );
		XmStringFree ( xmstr );
	}
	else
	{
		XmString xmstr = XmStringCreateSimple ( "Time sync. OFF" );
		XtVaSetValues ( _syncLength, XmNset, XmUNSET, XmNlabelString, xmstr, NULL );
		XmStringFree ( xmstr );
	}
	
	_linkViews = XtCreateManagedWidget ( NULL, xmToggleButtonWidgetClass, _rc, args, n );
	XtAddCallback ( _linkViews, XmNvalueChangedCallback, &ChoicesCmd::linkViewsCallback, ( XtPointer ) this );

	if ( mixwaveResources->linkViews() == True )
		XtVaSetValues ( _linkViews, XmNset, XmSET, NULL );
	else
		XtVaSetValues ( _linkViews, XmNset, XmUNSET, NULL );

	if ( mixwaveResources->syncTemplateLength() == False )
	{
		XmString xmstr = XmStringCreateSimple ( "Sync. mode disabled" );
		XtVaSetValues ( _linkViews, XmNlabelString, xmstr, NULL );
		XmStringFree ( xmstr );
		XtVaSetValues ( _linkViews, XmNindicatorOn, XmINDICATOR_NONE, NULL );
	}
	else
	{	
		if ( mixwaveResources->linkViews() == True )
		{
			XmString xmstr = XmStringCreateSimple ( "Template -> wave" );
			XtVaSetValues ( _linkViews, XmNlabelString, xmstr, NULL );
			XmStringFree ( xmstr );
		}
		else
		{
			XmString xmstr = XmStringCreateSimple ( "Wave -> Template" );
			XtVaSetValues ( _linkViews, XmNlabelString, xmstr, NULL );
			XmStringFree ( xmstr );
		}
	}
	
	_syncSelections = XtCreateManagedWidget ( "Sync. selections", xmToggleButtonWidgetClass, _rc, args, n );
	XtAddCallback( _syncSelections, XmNvalueChangedCallback, &ChoicesCmd::syncSelectionsCallback, ( XtPointer ) this );

	if ( mixwaveResources->syncSelections() == True )
		XtVaSetValues ( _syncSelections, XmNset, XmSET, NULL );
	else
		XtVaSetValues ( _syncSelections, XmNset, XmUNSET, NULL );

	_mixToCurrent = XtCreateManagedWidget ( "MixToCurrentWave", xmToggleButtonWidgetClass, _rc, args, n );
	XtAddCallback( _mixToCurrent, XmNvalueChangedCallback, &ChoicesCmd::mixToCurrentCallback, ( XtPointer ) this );

	if ( mixwaveResources->mixToCurrent() == True )
		XtVaSetValues ( _mixToCurrent, XmNset, XmSET, NULL );
	else
		XtVaSetValues ( _mixToCurrent, XmNset, XmUNSET, NULL );

	_remix = XtCreateManagedWidget ( "Enable remix", xmToggleButtonWidgetClass, _rc, args, n );
	XtAddCallback( _remix, XmNvalueChangedCallback, &ChoicesCmd::remixCallback, ( XtPointer ) this );

	if ( mixwaveResources->enableRemix() == True )
		XtVaSetValues ( _remix, XmNset, XmSET, NULL );
	else
		XtVaSetValues ( _remix, XmNset, XmUNSET, NULL );

	_messages = XtCreateManagedWidget ( "Enable message popups", xmToggleButtonWidgetClass, _rc, args, n );
	XtAddCallback ( _messages, XmNvalueChangedCallback, &ChoicesCmd::messagesCallback, ( XtPointer ) this );

	if ( mixwaveResources->enableMessagePopups() == True )
		XtVaSetValues ( _messages, XmNset, XmSET, NULL );
	else
		XtVaSetValues ( _messages, XmNset, XmUNSET, NULL );

	_warnings = XtCreateManagedWidget ( "Enable warning popups", xmToggleButtonWidgetClass, _rc, args, n );
	XtAddCallback ( _warnings, XmNvalueChangedCallback, &ChoicesCmd::warningsCallback, ( XtPointer ) this );

	if ( mixwaveResources->enableWarningPopups() == True )
		XtVaSetValues ( _warnings, XmNset, XmSET, NULL );
	else
		XtVaSetValues ( _warnings, XmNset, XmUNSET, NULL );
	
	_save = XtCreateManagedWidget ( "Save on exit", xmToggleButtonWidgetClass, _rc, args, n );
	XtAddCallback ( _save, XmNvalueChangedCallback, &ChoicesCmd::saveCallback, ( XtPointer ) this );

	if ( mixwaveResources->isSaveOnExit() == True )
		XtVaSetValues ( _save, XmNset, XmSET, NULL );
	else
		XtVaSetValues ( _save, XmNset, XmUNSET, NULL );

	_quit = XtCreateManagedWidget ( "Quit", xmPushButtonWidgetClass, _rc, NULL, 0 );
	XtAddCallback ( _quit, XmNactivateCallback, &ChoicesCmd::quitCallback, ( XtPointer ) this );

	// cater for WM 'X' pressed
	WM_DELETE_WINDOW = XmInternAtom ( XtDisplay ( _popup ), "WM_DELETE_WINDOW", False );
	XmAddWMProtocols ( _popup, &WM_DELETE_WINDOW, 1 );
	XmAddWMProtocolCallback ( _popup, WM_DELETE_WINDOW, quitCallback, ( XtPointer ) this );

	_popupRealised = True;
	XtPopup ( _popup, ( XtGrabKind ) XtGrabNone );
	XtManageChild ( _popup );
}


void ChoicesCmd::clientMessageProc ( Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;

	if ( cbs->reason != XmCR_CANCEL && cbs->reason != XmCR_OK )
	{
		obj->quit();
		XtDestroyWidget ( w );
	}
}


void ChoicesCmd::quitCallback ( Widget w, XtPointer clientData, XtPointer )
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;

	obj->quit();
	XtDestroyWidget ( w );
}


void ChoicesCmd::quit()
{
	_popupRealised = False;
	XtPopdown ( _popup );
}


void ChoicesCmd::syncLengthCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;
	
	obj->syncLength ( callData );
}


void ChoicesCmd::syncLength ( XtPointer callData )
{
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( cbs->set == XmSET )
	{
		mixwaveResources->syncTemplateLength ( True );
		XmString xmstr = XmStringCreateSimple ( "Time sync.  ON" );
		XtVaSetValues ( _syncLength, XmNlabelString, xmstr, NULL );
		XmStringFree ( xmstr );		

		XtVaSetValues ( _linkViews, XmNindicatorOn, True, NULL );

		if ( mixwaveResources->linkViews() == True )
		{
			XmString xmstr = XmStringCreateSimple ( "Template -> wave" );
			XtVaSetValues ( _linkViews, XmNlabelString, xmstr, NULL );
			XmStringFree ( xmstr );
		}
		else
		{
			XmString xmstr = XmStringCreateSimple ( "Wave -> Template" );
			XtVaSetValues ( _linkViews, XmNlabelString, xmstr, NULL );
			XmStringFree ( xmstr );
		}
	}
	else
	{
		mixwaveResources->syncTemplateLength ( False );
		XmString xmstr1 = XmStringCreateSimple ( "Time sync. OFF" );
		XtVaSetValues ( _syncLength, XmNlabelString, xmstr1, NULL );
		XmStringFree ( xmstr1 );

		XmString xmstr2 = XmStringCreateSimple ( "Sync. mode disabled" );
		XtVaSetValues ( _linkViews, XmNlabelString, xmstr2, NULL );
		XmStringFree ( xmstr2 );
		
		XtVaSetValues ( _linkViews, XmNindicatorOn, XmINDICATOR_NONE, NULL );
	}
}


void ChoicesCmd::linkViewsCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( mixwaveResources->syncTemplateLength() == True )
		obj->linkViews ( callData );
	else
	{
		if ( cbs->set == XmSET )
			XtVaSetValues ( obj->_linkViews, XmNset, XmUNSET, NULL );
		else
			XtVaSetValues ( obj->_linkViews, XmNset, XmSET, NULL );
	}
}


void ChoicesCmd::linkViews ( XtPointer callData )
{
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( cbs->set == XmSET )
	{
		mixwaveResources->linkViews ( True );
		XmString xmstr = XmStringCreateSimple ( "Template -> wave" );
		XtVaSetValues ( _linkViews, XmNlabelString, xmstr, NULL );
		XmStringFree ( xmstr );		
	}
	else
	{
		mixwaveResources->linkViews ( False );
		XmString xmstr = XmStringCreateSimple ( "Wave -> template" );
		XtVaSetValues ( _linkViews, XmNlabelString, xmstr, NULL );
		XmStringFree ( xmstr );
	}
}


void ChoicesCmd::messagesCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;
	
	obj->messages ( callData );
}


void ChoicesCmd::messages ( XtPointer callData )
{
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( cbs->set == XmSET )
		mixwaveResources->enableMessagePopups ( True );
	else
		mixwaveResources->enableMessagePopups ( False );
}


void ChoicesCmd::warningsCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;
	
	obj->warnings ( callData );
}


void ChoicesCmd::warnings ( XtPointer callData )
{
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( cbs->set == XmSET )
		mixwaveResources->enableWarningPopups ( True );
	else
		mixwaveResources->enableWarningPopups ( False );
}


void ChoicesCmd::saveCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;
	
	obj->save ( callData );
}


void ChoicesCmd::save ( XtPointer callData )
{
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( cbs->set == XmSET )
		mixwaveResources->setSaveOnExit ( True );
	else
		mixwaveResources->setSaveOnExit ( False );
}


void ChoicesCmd::syncSelectionsCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( cbs->set == XmSET )
		XtVaSetValues ( obj->_mixToCurrent, XmNset, XmUNSET, NULL );

	obj->syncSelections ( callData );
}


void ChoicesCmd::syncSelections ( XtPointer callData )
{
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( cbs->set == XmSET )
	{
		mixwaveResources->syncSelections ( True );
		mixwaveResources->mixToCurrent ( False );
	}
	else
		mixwaveResources->syncSelections ( False );
		
}


void ChoicesCmd::remixCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;
	
	obj->remix ( callData );
}


void ChoicesCmd::remix ( XtPointer callData )
{
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( cbs->set == XmSET )
		mixwaveResources->enableRemix ( True );
	else
		mixwaveResources->enableRemix ( False );
}


void ChoicesCmd::mixToCurrentCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	ChoicesCmd *obj = ( ChoicesCmd * ) clientData;
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;

	if ( cbs->set == XmSET )
	{
		if ( strlen ( mixwaveResources->currentWave() ) == 0 )
		{
			char msg[46];
			strncpy ( msg, "No current wave set in Options/ Wave control", sizeof ( msg ) );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			XtVaSetValues ( obj->_mixToCurrent, XmNset, XmUNSET, NULL );
			return;
		}
		
		XtVaSetValues ( obj->_syncSelections, XmNset, XmUNSET, NULL );
	}
	
	obj->mixToCurrent ( callData );
}


void ChoicesCmd::mixToCurrent ( XtPointer callData )
{
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;
	
	if ( cbs->set == XmSET )
	{
		mixwaveResources->mixToCurrent ( True );
		mixwaveResources->syncSelections ( False );
	}
	else
		mixwaveResources->mixToCurrent ( False );
}
