/*
 MenuBar.cc - menu bar whose panes support items executing commands

 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.
*/

#include "MenuBar.h"
#include "Cmd.h"
#include "CmdList.h"
#include "ButtonInterface.h"
#include "ToggleInterface.h"
#include "SeparatorInterface.h"

#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>


MenuBar::MenuBar ( const char *name, Widget parent ) : Interface ( name )
{
	_w = XmCreateMenuBar ( parent, _name, NULL, 0 );	// parent is Motif widget
	installDestroyHandler();
}


void MenuBar::createPulldown ( Widget parent, CmdList *list )
{
	int i;
	Widget pulldown, cascade;
	
	// create pulldown menu pane for this list of commands
	pulldown = XmCreatePulldownMenu ( parent, ( char * ) list->name(), NULL, 0 );
	
	// each menubar entry has a cascade button from which the pane is pulled down
	cascade = XtVaCreateManagedWidget ( ( char * ) list->name(),
						xmCascadeButtonWidgetClass, parent,
						XmNsubMenuId, pulldown, NULL );
	// create menu entry for each item on CmdList - see 'Cmd.h' for types
	// needs rewriting as 'case'
	for ( i = 0; i < list->size(); i++ )
	{
		if ( ( *list )[i]->type() == Cmd::myList )
			createPulldown ( pulldown, ( CmdList * ) ( *list )[i] );
		else
			if ( ( *list )[i]->type() == Cmd::Action )
			{
				CmdInterface *ci;
				ci = new ButtonInterface ( pulldown,  ( *list )[i] );
				ci->manage();
			}
			else
				if ( ( *list )[i]->type() == Cmd::Toggle )
				{
					CmdInterface *ci;
					ci = new ToggleInterface ( pulldown,  ( *list )[i] );
					ci->manage();
				}
				else				
					if ( ( *list )[i]->type() == Cmd::Separator )
					{
						CmdInterface *ci;
						ci = new SeparatorInterface ( pulldown,  ( *list )[i] );
						ci->manage();
					}
	}
}


void MenuBar::addCommands ( CmdList *list )
{
	createPulldown ( _w, list );
}
